from .binsx import BinSX
from .coomer import Coomer
from .ephemeride import Ephemeride
from .gotanynudes import Gotanynudes
from .leakedzone import Leakedzone
from.mediafire import MediaFire
from .openfoodfacts_v3 import OpenFoodFactsV3
from .voiranime import VoirAnime
from .voirdrama import VoirDrama



from kelian.utils import fix_encoding
from fake_useragent import UserAgent
import requests
import html


class Ephemeride():
    name = "Ephemeride"
    domain = "https://www.ephemeride.com/"

    def __init__(self):
        self.domain = "https://www.ephemeride.com/free/"

    def __request__(self, url:str) -> str:
        res = requests.get(self.domain + url).content.decode("iso-8859-1")
        return fix_encoding(html.unescape(res)).replace("\\'", "'").replace("\\\"", "\"")
    
    def get_fete_du_jour(self) -> str:
        content = self.__request__("fete.jsp")
        content = content.split("class=\"FeteHomme\">")
        return content[1].split("</font>")[0]
    
    def get_evenement_du_jour(self) -> str:
        content = self.__request__("evenement.jsp")
        content = content.split("<td class=\"fmb\"")[1]
        i = content.index(">") + 1
        content = content[i:].split("<br>")[0]
        content = content.replace("<b>", "").replace("</b>", "")
        return content
    
    def get_diction_du_jour(self) -> str:
        content = self.__request__("dicton.jsp")
        content = content.split("<td class=\"fmb\"")[1]
        i = content.index(">") + 1
        content = content[i:].split("<br>")[0]
        content = content.replace("<b>", "").replace("</b>", "")
        return content
    
    def get_citation_du_jour(self) -> tuple[str, str]:
        content = self.__request__("citation.jsp")
        content = content.split("<td class=\"fmb\"")[1]
        i = content.index(">") + 1
        content = content[i:].split("<br>")[0]
        content = content.replace("<b>", "").replace("</b>", "")
        p = content.split("\"cita\">")[1].split("</span>")[0]
        return content.split("<div")[0], p
    
    def get_proverbe_du_jour(self) -> str:
        content = self.__request__("proverbe.jsp")
        content = content.split("<td class=\"fmb\"")[1]
        i = content.index(">") + 1
        content = content[i:].split("<br>")[0]
        content = content.replace("<b>", "").replace("</b>", "")
        return content
    
    def get_jour(self) -> dict:
        res = requests.get(
            "https://www.ephemeride.com/home/1/calendriers-et-histoire.html", 
            headers={ "User-Agent": UserAgent().firefox })
        content = fix_encoding(html.unescape(res.content.decode("iso-8859-1")))
        content = content.replace("\\'", "'").replace("\\\"", "\"")
        td = content.split("class=\"TexteGrisClair\"")
        td1 = td[1].split("<br>")
        td2 = td[2].split("<br>")
        f = lambda x: x.replace("\r", "").replace("\n", "").replace("\t", "")
        return {
            "day_of_the_year": int(f(td1[1]).split("e")[0]),
            "days_remaining": int(f(td1[2]).split(" ")[0]),
            "week": int(f(td1[3]).split("e")[0]), 
            "hour": f(td2[1]).replace("<b>", "").replace("</b>", ""), 
            "timezone": f(td2[2]), 
            "type_of_time": f(td2[3]).split("<")[0]
        }



from fake_useragent import UserAgent
import requests


class OpenFoodFactsV3:
    name = "Open Food Facts (API V3)"
    domain = "https://fr.openfoodfacts.org/"

    def __init__(self):
        self.domain = "https://fr.openfoodfacts.org/api/v3/"

    def get_products(self, barcode:str, 
                           product_type:str=None, 
                           cc:str=None, 
                           lc:str=None, 
                           tags_lc:str=None, 
                           fields:str=None, 
                           knowledge_panels_included:str=None, 
                           knowledge_panels_excluded:str=None) -> dict:
        """
        Get Product Data
        ----------------

        Retrieve information for a product with a specific barcode.

        The fields parameter allows to specify what fields to retrieve.

        **product_type**: Used for READ queries for one product. Expected product type of the requested product. 
            Defaults to the product type of the server the query is sent to (e.g. 'food' for Open Food Facts, 'beauty' for Open Beauty Facts, etc.). 'all' matches all product types. 
            If the product exists on a different server that matches the requested product type, the API will return a 302 redirect to the correct server. 
            Otherwise, the API will return a 404 error. It is possible that new product types will be added in the future.
            **Allowed**: all | beauty | food | petfood | product

        **cc**: 2 letter code of the country of the user. Used for localizing some fields in returned values (e.g. knowledge panels). 
            If not passed, the country may be inferred by the IP address of the request.
            **Examples**: us

        **lc**: 2 letter code of the language of the user. Used for localizing some fields in returned values (e.g. knowledge panels).
            If not passed, the language may be inferred by the Accept-Language header of the request.
            **Examples**: fr

        **tags_lc**: 2 letter language code to request names of tags in a specific language. For READ requests:
            if passed, all taxonomized tags of the response will include a lc_name property with the translation in the requested language,
            if available. Otherwise, the property value will contain the name in the original language, prefixed by the 2 language code and a colon.

        **fields**: Comma separated list of fields requested in the response. Special values:
                - `none`: returns no fields
                - `raw`: returns all fields as stored internally in the database
                - `all`: returns all fields except generated fields that need to be explicitly requested such as "knowledge_panels".
            Defaults to "all" for READ requests. The "all" value can also be combined with fields like "attribute_groups" and "knowledge_panels".
        
        **knowledge_panels_included**: When knowledge_panels are requested, you can specify which panels should be in the response.
            All the others will be excluded.
            **Examples**: health_card, environment_card
        
        **knowledge_panels_excluded**: When knowledge_panels are requested, you can specify which panels to exclude from the response.
            All the others will be included. If a panel is both excluded and included (with the knowledge_panels_excluded parameter), it will be excluded.
            **Examples**: health_card, environment_card 
        """

        params = {}
        if product_type is not None: params["product_type"] = product_type
        if cc is not None: params["cc"] = cc
        if lc is not None: params["lc"] = lc
        if tags_lc is not None: params["tags_lc"] = tags_lc
        if fields is not None: params["fields"] = fields
        if knowledge_panels_included is not None: params["knowledge_panels_included"] = knowledge_panels_included
        if knowledge_panels_excluded is not None: params["knowledge_panels_excluded"] = knowledge_panels_excluded

        return requests.get(
            self.domain + "product/" + barcode, 
            headers={
                "User-Agent": UserAgent().firefox
            },
            params=params
        ).json()
    
    def get_list_of_preference_importance_values(self) -> dict:
        """
        Get List of Preference Importance Values
        ----------------------------------------

        These parameters are used to compute the product preferences score.

        for an overview see [Explanation on Product Attributes](https://openfoodfacts.github.io/documentation/docs/Product-Opener/api/explain-product-attributes/)"
        """
        return requests.get(
            self.domain + "preferences", 
            headers={
                "User-Agent": UserAgent().firefox
            }
        ).json()

    def get_list_of_attribute_groups_and_attributes(self) -> dict:
        """
        Get List of Attribute Groups and Attributes
        -------------------------------------------

        for an overview see [Explanation on Product Attributes](https://openfoodfacts.github.io/documentation/docs/Product-Opener/api/explain-product-attributes/)"
        """
        return requests.get(
            "/".join(self.domain.split("/")[:-2]) + "/v3.4/attribute_groups", 
            headers={
                "User-Agent": UserAgent().firefox
            }
        ).json()

    def get_canonical_tags_for_a_list_of_local_tags(self, tagtype:str, local_tags_list:str, lc:str=None) -> dict:
        """
        Get canonical tags for a list of local tags
        -------------------------------------------

        **tagtype**: The type of taxonomy to canonicalize tags for (e.g., ingredients, categories).
            **Examples**: ingredients
        
        **local_tags_list**: A comma-separated list of local tags to canonicalize.
            **Examples**: sucre,eau

        **lc**: 2-letter code of the language of the user. Used for localizing some fields in returned values. If not passed, the language may be inferred by the subdomain of the request.
            **Examples**: fr
        """
        params = {
            "tagtype": tagtype, 
            "local_tags_list": local_tags_list
        }
        if lc is not None: params["lc"] = lc
        return requests.get(
            self.domain + "taxonomy_canonicalize_tags", 
            headers={
                "User-Agent": UserAgent().firefox
            },
            params=params
        ).json()
    
    def get_display_tags_in_a_specific_language_for_a_list_of_taxonomy_tags(self, tagtype:str,  canonical_tags_list:str, lc:str=None) -> dict:
        """
        Get display tags in a specific language for a list of taxonomy tags
        -------------------------------------------------------------------

        **tagtype**: The type of taxonomy to retrieve display tags for (e.g., ingredients, categories).
            **Examples**: ingredients
        
        **canonical_tags_list**: A comma-separated list of canonical taxonomy tags to retrieve display tags for.
            **Examples**: en:sugar,en:water

        **lc**: 2-letter code of the language to return display tags in.
            **Examples**: fr
        """
        params = {
            "tagtype": tagtype, 
            "canonical_tags_list": canonical_tags_list
        }
        if lc is not None: params["lc"] = lc
        return requests.get(
            self.domain + "taxonomy_display_tags", 
            headers={
                "User-Agent": UserAgent().firefox
            },
            params=params
        ).json()
    
    def get_taxonomy_suggestions(self, tagtype:str=None, 
                                       cc:str=None, 
                                       lc:str=None, 
                                       string:str=None, 
                                       categories:str=None, 
                                       shape:str=None, 
                                       limit:str=None, 
                                       get_synonyms:str=None, 
                                       term:str=None) -> dict:
        """
        Get Taxonomy Suggestions
        ------------------------

        Open Food Facts uses multilingual [taxonomies](https://wiki.openfoodfacts.org/Taxonomies_introduction) to normalize entries for categories, labels, ingredients, packaging shapes / materials / recycling instructions and many more fields.

        This API returns taxonomy entries suggestions that can be used in product edit forms, search forms etc. (for instance in autocomplete dropdowns using libraries like Tagify or select2 on the Web).

        Suggestions filtering:

        The string parameter allows to get only suggestions that contain a specific string (useful for autocomplete suggestions).

        Suggestions ordering:

        - For packaging shapes and materials, suggestions are ordered first by the number of packaging components they appear in (restricted by country, categories and shape (for materials) if they are passed as parameters).
        - for all other taxonomies, results are ordered alphabetically

        If a string is passed, an additional sort is done to put first suggestions that start with the string, followed by suggestions with a word that start with the string, and then suggestions that contain the string anywhere.
        
        **tagtype**: 
            **Examples**: additives
        
        **cc**: 2 letter code of the country of the user. Used for localizing some fields in returned values (e.g. knowledge panels). If not passed, the country may be inferred by the IP address of the request.
            **Examples**: us
        
        **lc**: 2 letter code of the language of the user. Used for localizing some fields in returned values (e.g. knowledge panels). If not passed, the language may be inferred by the Accept-Language header of the request.
            **Examples**: fr
        **string**: Optional string used to filter suggestions (useful for autocomplete). If passed, suggestions starting with the string will be returned first, followed by suggestions matching the string at the beginning of a word, and suggestions matching the string inside a word.
            **Examples**: pe
        
        **categories**: Comma separated list of categories tags (e.g. "en:fats,en:unsalted-butters" or categories names in the language indicated by the "lc" field (e.g. "graisses, beurres salés" in French)
            **Examples**: yougurts
        
        **shape**: Shape of packaging component (tag identified in the packaging_shapes taxonomy, or plain text tag name in the language indicated by the "lc" field).
            **Examples**: bottle
        
        **limit**: Maximum number of suggestions. Default is 25, max is 400.
        
        **get_synonyms**: Whether or not to include "matched_synonyms" in the response. Set to 1 to include.
        
        **term**: Alias for the "string" parameter provided for backward compatibility. "string" takes precedence.
        """
        params = {}
        if tagtype is not None: params["tagtype"] = tagtype
        if cc is not None: params["cc"] = cc
        if lc is not None: params["lc"] = lc
        if string is not None: params["string"] = string
        if categories is not None: params["categories"] = categories
        if shape is not None: params["shape"] = shape
        if limit is not None: params["limit"] = limit
        if get_synonyms is not None: params["get_synonyms"] = get_synonyms
        if term is not None: params["term"] = term

        return requests.get(
            self.domain + "taxonomy_suggestions", 
            headers={
                "User-Agent": UserAgent().firefox
            },
            params=params
        ).json()

    def get_tag_knowledge_panels(self, tagtype:str, tag_or_tagid:str, cc:str=None, lc:str=None) -> dict:
        """
        Get Tag Knowledge Panels
        ------------------------

        Return knowledge panels for a tag.

        Currently the knowledge panels returned are:

        Categories:

        - Packaging stats for a category

        **tagtype**: Type of the tag
            **Examples**: categories

        **tag_or_tagid**: Tag name (e.g. yogurts) or tag id (e.g. en:yogurts)

        **cc**: 2 letter code of the country of the user. Used for localizing some fields in returned values (e.g. knowledge panels).
            If not passed, the country may be inferred by the IP address of the request.
            **Examples**: us

        **lc**: 2 letter code of the language of the user. Used for localizing some fields in returned values (e.g. knowledge panels).
            If not passed, the language may be inferred by the Accept-Language header of the request.
            **Examples**: fr 
        """
        params = {}
        if cc is not None: params["cc"] = cc
        if lc is not None: params["lc"] = lc

        return requests.get(
            self.domain + f"tag/{tagtype}/{tag_or_tagid}", 
            headers={
                "User-Agent": UserAgent().firefox
            },
            params=params
        ).json()



from fake_useragent import UserAgent
from bs4 import BeautifulSoup
import requests


class BinSX():
    name = "Bin.sx"
    domain = "https://paste.bin.sx"

    def __init__(self):
        self.domain = "https://paste.bin.sx/"

    def get_text_by_id(self, id:str) -> str:
        content = requests.get(
            self.domain + "index.php?p=" + id, 
            headers={
                "User-Agent": UserAgent().firefox, 
                "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8", 
                "Accept-Language": "fr,fr-FR;q=0.8,en-US;q=0.5,en;q=0.3", 
                "Accept-Encoding": "gzip, deflate, br, zstd", 
                "Connection": "keep-alive", 
                "Upgrade-Insecure-Requests": "1", 
                "Sec-Fetch-Dest": "document", 
                "Sec-Fetch-Mode": "navigate", 
                "Sec-Fetch-Site": "none", 
                "Sec-Fetch-User": "?1", 
                "DNT": "1", 
                "Sec-GPC": "1", 
                "Priority": "u=0, i", 
                "Pragma": "no-cache", 
                "Cache-Control": "no-cache", 
            }
        ).content.decode("utf8")
        soup = BeautifulSoup(content, "html.parser")
        box = soup.find("textarea", { "id": "textbox"})
        return box.getText()



import requests
from datetime import datetime
from os.path import join as join_path
from kelian import ProgressBar


class MediaFire():
    name = "MediaFire"
    domain = "https://www.mediafire.com"

    def __init__(self, url_or_code:str) -> None:
        self.domain = "https://www.mediafire.com/file/"
        self.url = url_or_code if url_or_code.startswith("http") else self.domain + url_or_code
        self.content = None

    def __request__(self) -> None:
        if self.content is None:
            self.content = requests.get(
                self.url, 
                headers={
                    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:144.0) Gecko/20100101 Firefox/144.0"
                }
            ).content.decode("utf8")

    def __convert_size__(self, size_str: str) -> int:
        units = {"B": 1, "KB": 1024, "MB": 1024**2, "GB": 1024**3, "TB": 1024**4}
        size_str = size_str.strip().upper()
        for unit in sorted(units.keys(), key=len, reverse=True):
            if size_str.endswith(unit):
                number = size_str[: -len(unit)]
                return int(float(number) * units[unit])
        raise ValueError(f"Unité inconnue dans la chaîne: {size_str}")

    def get_info(self) -> dict:
        self.__request__()
        fn = self.content.split("<div class=\"filename\">")[1].split("</div>")[0]
        fs = self.content.split("<ul class=\"details\">")[1].split("</span>")[0].split("<span>")[1]
        d = self.content.split("<div class=\"DLExtraInfo-sectionDetails\">")[1].split("</p>")[0].split("<p>")[1]
        ud = datetime.strptime(d.split(" from ")[1].split(" on ")[1], "%B %d, %Y at %I:%M %p")
        uc = d.split(" from ")[1].split(" on ")[0]
        return {"filename": fn, "size": self.__convert_size__(fs), "uploaded date": ud, "uploaded contry": uc}
    
    def download(self, output:str, chunk_size:int=8192) -> None:
        self.__request__()
        url = self.content.split("aria-label=\"Download file\"")[1].split("href=\"")[1].split("\"")[0]
        with requests.get(url, stream=True) as r:
            r.raise_for_status()
            total_size = self.get_info().get("size")
            pb = ProgressBar(total_size, enabled_average_time=False)
            pb.format("Download file [")
            with open(join_path(output, self.get_info().get("filename")), 'wb') as f:
                for chunk in r.iter_content(chunk_size=chunk_size):
                    f.write(chunk)
                    pb.current += chunk_size
                    print(pb.display(), end="\r")



from types import GeneratorType

class Scraper:
    name = "Scraper"

    def __init__(self):
        pass

    def __yield_info__(self, **kwargs):
        gens = {k: v for k, v in kwargs.items() if isinstance(v, GeneratorType)}
        static = {k: v for k, v in kwargs.items() if not isinstance(v, GeneratorType)}

        while gens:
            data = static.copy()
            remove_keys = []
            for key, gen in gens.items():
                try:
                    data[key] = next(gen)
                except StopIteration:
                    remove_keys.append(key)
            # enlever les générateurs épuisés
            for k in remove_keys:
                gens.pop(k)
            # ⚠️ ne yield que si on a encore au moins une valeur générée
            if len(data) > len(static):
                yield data

    def search(self, text:str, by_regex:bool=True) -> list:
        """Méthode pour rechercher un profil"""
        message = "[ERROR] Search method is not coded"
        print(message)

    def get_posts(self, id:str) -> list[dict]:
        """Récupérer les posts du profil"""
        message = "[ERROR] Get_posts method is not coded"
        print(message)

    def download(self, posts:list[dict]) -> None:
        """Télécharger les posts"""
        message = "[ERROR] Download method is not coded"
        print(message)



from fake_useragent import UserAgent
from bs4 import BeautifulSoup
import requests
import json


class VoirDrama():
    name = "Voir Drama"
    domain = "https://voirdrama.org/"

    def __init__(self):
        self.domain = "https://voirdrama.org/wp-admin"

    def __search__(self, data:str) -> requests.Response:
        content = requests.post(
            self.domain + "/admin-ajax.php", 
            data=data, 
            headers={
                "User-Agent": UserAgent().firefox, 
                "Accept": "text/html", 
                "Accept-Language": "fr,fr-FR;q=0.8,en-US;q=0.5,en;q=0.3", 
                "Accept-Encoding": "gzip, deflate, br, zstd", 
                "Content-type": "application/x-www-form-urlencoded; charset=UTF-8", 
                "Origin": VoirDrama.domain, 
                "Connection": "keep-alive", 
                "Referer": VoirDrama.domain, 
                "Sec-Fetch-Dest": "empty", 
                "Sec-Fetch-Mode": "cors", 
                "Sec-Fetch-Site": "same-origin", 
                "DNT": "1", 
                "Sec-GPC": "1", 
            }
        ).content.decode("utf8")

        data = content.split("___ASPSTART_DATA___")[1]
        data = data.split("___ASPEND_DATA___")[0]

        return json.loads(data)

    def search(self, text:str, vf:bool=True, vostfr:bool=True) -> list:
        result:list[dict] = []

        if vostfr:
            vostfrdata = self.__search__(f"action=ajaxsearchpro_search&aspp={text}&asid=7&asp_inst_id=7_2&options=aspf%5Bvf__1%5D%3Dvf%26asp_gen%5B%5D%3Dexcerpt%26asp_gen%5B%5D%3Dcontent%26asp_gen%5B%5D%3Dtitle%26filters_initial%3D1%26filters_changed%3D0%26qtranslate_lang%3D0%26current_page_id%3D11")
            result += vostfrdata["results"]

        if vf:
            vfdata = self.__search__(f"action=ajaxsearchpro_search&aspp={text}&asid=6&asp_inst_id=6_2&options=aspf%5Bvf__1%5D%3Dvf%26asp_gen%5B%5D%3Dexcerpt%26asp_gen%5B%5D%3Dcontent%26asp_gen%5B%5D%3Dtitle%26filters_initial%3D1%26filters_changed%3D0%26qtranslate_lang%3D0%26current_page_id%3D11")
            result += vfdata["results"]
        
        return result
    
    def get_drama(self, link:str) -> dict:
        result = {}
        # requête http
        content = requests.get(
            link, 
            headers={
                "User-Agent": UserAgent().firefox, 
                "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8", 
                "Accept-Language": "fr,fr-FR;q=0.8,en-US;q=0.5,en;q=0.3", 
                "Accept-Encoding": "gzip, deflate, br, zstd", 
                "Referer": VoirDrama.domain, 
                "Connection": "keep-alive", 
                "Upgrade-Insecure-Requests": "1", 
                "Sec-Fetch-Dest": "document", 
                "Sec-Fetch-Mode": "navigate", 
                "Sec-Fetch-Site": "same-origin", 
                "Sec-Fetch-User": "?1", 
                "DNT": "1", 
                "Sec-GPC": "1", 
                "Priority": "u=0, i", 
                "Pragma": "no-cache", 
                "Cache-Control": "no-cache", 
            }
        ).content.decode("utf8")
        # transformation du contenant de la page html en objet
        soup = BeautifulSoup(content, "html.parser")
        # récupération de l'image
        result["image"] = soup.find("div", class_="site-content").find("img").get("src")
        # box info
        info = soup.find("div", class_="post-content")
        # récupération de la note
        result["rate"] = float(info.find("div", class_="post-rating").find("span").text)
        # récupération des informations
        for infoline in info.find_all("div", class_="post-content_item"):
            key = infoline.find("div", class_="summary-heading").getText().strip()
            value = infoline.find("div", class_="summary-content").getText().strip()
            result[key] = eval(value) if value.isnumeric() else value
        # récupération de la deuxième partie de la page html
        part2 = soup.find("div", class_="main-col-inner")
        # récupération du résumé
        result["Synopsis"] = part2.find("div", class_="description-summary").getText().strip()
        # récupération de la liste des épisodes
        result["episode list"] = []
        for li in part2.find_all("li", class_="wp-manga-chapter"):
            result["episode list"].append({
                "name": li.find("a").getText().strip(), 
                "link": li.find("a").get("href"), 
                "upload date": li.find("span").getText().strip()
            })

        return result



from setuptools import setup, find_packages

setup(
    name="pycalypso",
    version="0.1.9",
    packages=find_packages(),
    install_requires=[],
    author="Game_K",
    author_email="game_k@laposte.net",
    description="Custom Python library for web scraping, flexible and tailored to your projects",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/Game-K-Hack/calypso",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
)
